var Capture = new Object();

/* ==================== Capturing ==================== */

Capture._currentDoc = null;
Capture._capturedNode = null;
Capture._capturing = false;
Capture._highlighting = false;

Capture.onDocumentChanged = function() {
    if (Capture._capturing) {
        Capture._stopCapturingXPath(Capture._currentDoc);
    }
    
    Capture.clearAllHighlighting();
    
    Capture._currentDoc = null;
    Capture._capturedNode = null;
}

Capture.captureXPath = function() {
    var doc = Code.instrumentContentDocument();

    Capture._currentDoc = doc;
    Capture._addCapturingListeners(doc);
    
    document.getElementById("solvent-scrapeToolbar-code-captureTab-capture-button").disabled = true;
    
    Capture._capturing = true;
}

Capture.upCapturedNode = function() {
    var textbox = Capture.getXPathTextbox();
    var xpath = textbox.value;
    var i = xpath.lastIndexOf("/");
    if (i > 0) {
        textbox.value = xpath.substring(0, i);
        Capture.highlightXPath();
    }
}

Capture.highlightNode = function(node) {
    Highlighter.unhighlightStyle(Code.captureStyleClassName);
    
    Highlighter.highlight(node, Code.captureStyleClassName, function(m) {
        m.className = "highlighted";
    });
}

Capture.onXPathTextboxInput = function() {
    Capture.highlightXPath();
}

Capture._doneCapturingXPath = function() {
    var doc = Capture._currentDoc;
    Capture._stopCapturingXPath(doc);
    
    Capture.highlightXPath();
}

Capture._stopCapturingXPath = function(doc) {
    Capture._removeCapturingListeners(doc);
    
    document.getElementById("solvent-scrapeToolbar-code-captureTab-capture-button").disabled = false;
    
    Highlighter.unhighlightStyle(Code.captureStyleClassName);
    
    Capture._capturing = false;
}

Capture._captureNode = function(node) {
    var xpath = "";
    
    var namespace = node.ownerDocument.documentElement.namespaceURI;
    var prefix = namespace ? "x:" : "";
  
    var node2 = node;
    var doc = Capture._currentDoc;
    while (node2 && node2 != doc) {
        var tag = node2.tagName.toLowerCase();
        var id = node2.id;
        var className = node2.className;
    
        var segment = prefix + tag;
        if (tag == "tr") {
            var rowCount = node2.parentNode.rows.length;
            if (rowCount > 1 && rowCount < 5) {
                segment += '[' + (node2.rowIndex+1) + ']';
            }
        } else if (tag == "td") {
            var cellCount = node2.parentNode.cells.length;
            if (cellCount > 1 && cellCount < 5) {
                segment += '[' + (node2.cellIndex+1) + ']';
            }
        }
    
        if (className && className != "") {
            var classes = className.split(" ");
            var newClasses = [];
            for (var c = 0; c < classes.length; c++) {
                var cl = classes[c];
                if (cl.indexOf("simile-solvent-") < 0) {
                    newClasses.push(cl);
                }
            }
            if (newClasses.length > 0) {
                segment += '[@class="' + newClasses.join(" ") + '"]';
            }
        }
        
        if (id && id != "") {
            xpath = "//" + segment + '[@id="' + id + '"]' + xpath;
            break;
        }
      
        xpath = "/" + segment + xpath;
        
        node2 = node2.parentNode;
    }
    
    Capture.getXPathTextbox().value = xpath;
    
    Capture.highlightNode(node);
    DOM.constructTree([ node ]);
    
    Capture._capturedNode = node;
}

Capture._capturingMouseListener = {
    _captured : false,
    
    handleEvent: function(aEvent) {
        var elmt = aEvent.target;
        if (!this._captured && aEvent.type != "click" && elmt != Capture._capturedNode) {
            Capture._captureNode(elmt);
        }
    
        if (aEvent.type == "mousedown") {
            this._captured = true;
        } else if (aEvent.type == "click" ||
                  (aEvent.type == "mouseover" && this._captured)) {
                  
            aEvent.stopPropagation();
            aEvent.preventBubble();
            aEvent.preventDefault();
            aEvent.cancelBubble = true;
            Capture._doneCapturingXPath();
        }
    }
}

Capture._addCapturingListeners = function(doc) {
    Capture._capturingMouseListener._captured = false;
    doc.addEventListener("mousedown", Capture._capturingMouseListener, true);
    doc.addEventListener("mouseover", Capture._capturingMouseListener, true);
    doc.addEventListener("click", Capture._capturingMouseListener, true);
}

Capture._removeCapturingListeners = function(doc) {
    doc.removeEventListener("mousedown", Capture._capturingMouseListener, true);
    doc.removeEventListener("mouseover", Capture._capturingMouseListener, true);
    doc.removeEventListener("click", Capture._capturingMouseListener, true);
}


/* ==================== Highlighting ==================== */

Capture.clearAllHighlighting = function() {
    Highlighter.unhighlightStyle(Code.captureStyleClassName);
    Capture.clearXPathHighlighting();
    Capture._highlighting = false;
}

Capture.isHighlighting = function() {
    return Capture._highlighting;
}

Capture.highlightXPath = function(skipReconstructingDOM) {
    Capture.clearXPathHighlighting();
    
    var doc = Code.instrumentContentDocument();
    
    var xpath = Capture.getXPathTextbox().value;
    var nodes = [];
    try {
        nodes = Capture._getNodesOnXPath(doc, xpath);
        for (var i = 0; i < nodes.length; i++) {
            Highlighter.highlight(nodes[i], Code.highlightStyleClassName, function(m) {
                m.className = "captured";
            });
        }
        Capture.getXPathTextbox().className = "";
    } catch (e) {
        // if the xpath evaluation triggers an error, we signal it by changing the background
        // of the xpath text fields
        Capture.getXPathTextbox().className = "bad";
    }
  
    if (!skipReconstructingDOM) {
        DOM.constructTree(nodes);
    }
  
    Capture._highlighting = true;
};

Capture.clearXPathHighlighting = function() {
    Highlighter.unhighlightStyle(Code.highlightStyleClassName);
};

Capture._getNodesOnXPath = function(doc, xpath) {
    var nodes = [];
    var namespace = doc.documentElement.namespaceURI;
    var nsResolver = namespace ? function(prefix) {
        return (prefix == 'x') ? namespace : null;
    } : null;
    
    var iterator = doc.evaluate(xpath, doc, nsResolver, XPathResult.ANY_TYPE,null);
    var node = iterator.iterateNext();
    while (node) {
        nodes.push(node);
        node = iterator.iterateNext();
    }
    
    return nodes;
}

Capture.getXPathTextbox = function() {
    return document.getElementById("solvent-scrapeToolbar-code-captureTab-xpath-textbox");
}
