var Code = new Object();

Code._codeTabIndex = 1;
Code.lastSelectedTabIndex = 0;

Code._solventStyleID = "simile-solvent-style";
Code.highlightStyleClassName = "simile-solvent-highlight";
Code.captureStyleClassName = "simile-solvent-capture";

Code._unloadHandler = {
    handleEvent : function(aEvent) {
        //Solvent.debugPrint("Code._unloadHandler.handleEvent");
        Code._deinstrumentDocument();
    }
};

Code._instrumentedDoc = null;

Code.instrumentContentDocument = function() {
    //Solvent.debugPrint("Code.instrumentContentDocument");
    
    var tabBrowser = top.document.getElementById("content");
    var browser = tabBrowser.selectedBrowser;
    var doc = browser.contentDocument;
    
    if (Code._instrumentedDoc == doc) {
        return doc;
    }
    
    Code._deinstrumentDocument();
    
    var style = doc.getElementById(Code._solventStyleID);
    if (!style) {
        var head = doc.getElementsByTagName("head").item(0);
        if (head) {
            style = doc.createElement("style");
        } else {
            head = doc.getElementsByTagNameNS("http://www.w3.org/1999/xhtml", "head").item(0);
            style = doc.createElementNS("http://www.w3.org/1999/xhtml", "style");
        }
        
        style.id = Code._solventStyleID;
        style.innerHTML = 
          "." + Code.highlightStyleClassName + " { background-color: #ffff80; border: 1px solid #ff0000; }\n" +
          "." + Code.captureStyleClassName + " { background-color: #ffff80; border: 1px solid #0000ff; }";
        
        head.appendChild(style);
    }
    
    browser.addEventListener("beforeunload", Code._unloadHandler, false);
    
    Code._instrumentedDoc = doc;
    Code._instrumentedBrowser = browser;
    
    return doc;
}

Code._deinstrumentDocument = function() {
    //Solvent.debugPrint("Code._deinstrumentDocument");
    
    Capture.onDocumentChanged();
    DOM.onDocumentChanged();
    
    if (Code._instrumentedBrowser) {
        Code._instrumentedBrowser.removeEventListener("beforeunload", Code._unloadHandler, false);
    }
    Code._instrumentedDoc = null;
    Code._instrumentedBrowser = null;
}

Code.run = function() {
    var button = document.getElementById("solvent-scrapeToolbar-code-run-button");
    try {
        button.disabled = true;
        Runtime.run(
            Code.getCodeTextbox().value, 
            function() { 
                button.disabled = false;
                var tabs = document.getElementById("solvent-scrapeToolbar-code-rightPanel-tabs");
                if (Console.count > 0) {
                    tabs.selectedIndex = 1;
                } else if (Results.count > 0) {
                    tabs.selectedIndex = 2;
                } else {
                    alert("Finished. Your code produces no output.");
                }
            },
            function(e) {
                alert(e); 
                button.disabled = false;
            }
        );
    } catch (e) {
        Solvent.debugPrint(e);
        alert("Error running the code. Perhaps Piggy Bank is not installed?");
        button.disabled = false;
    }
}

Code.generateJavascript = function() {
    var code = DOM.generateJavascript();
    var textbox = Code.getCodeTextbox();
    if (textbox.value == "") {
        textbox.value = code;
    } else {
        var text = textbox.value;
        var selectionStart = Math.min(textbox.selectionStart, textbox.selectionEnd);
        var selectionEnd = Math.max(textbox.selectionStart, textbox.selectionEnd);
        
        var prefix = text.substr(0, selectionStart);
        var suffix = text.substr(selectionEnd);
        var insert = "\n" + code + "\n";
        
        textbox.value = prefix + insert + suffix;
        textbox.setSelectionRange(selectionStart, selectionStart + insert.length);
    }
    textbox.focus();
}

Code.insertTemplate = function(filename) {
    try {
        var file = Solvent.chromeURLToFile("chrome://solvent/content/scripts/" + filename);
        Code._insertCodeFile(file);
    } catch (e) {
        Solvent.debugPrint(e);
    }
}

Code.insertFromFile = function() {
    try {
        var nsIFilePicker = Components.interfaces.nsIFilePicker;
        var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
        
        fp.init(window, "Open", nsIFilePicker.modeOpen);
        fp.appendFilter("Javascript Files", "*.js");
        fp.appendFilters(nsIFilePicker.filterText | nsIFilePicker.filterAll);
        
        var r = fp.show();
        if (r == nsIFilePicker.returnOK) {
            var file = fp.file;
            Code._insertCodeFile(file);
        }
    } catch (e) {
        Solvent.debugPrint(e);
    }
}

Code._insertCodeFile = function(file) {
    var s = Solvent.readFromFile(file);
    
    var textbox = Code.getCodeTextbox();
    if (textbox.value.length > 0) {
        Code.addTab();
    }
    
    Code.getCodeTextbox().value = s;
}

Code.getCodeTextbox = function() {
    return document.getElementById("solvent-scrapeToolbar-leftPanel-tabbox").selectedPanel.codeTextbox;
}

Code.onRightPanelTabSelected = function() {
    document.getElementById('solvent-scrapeToolbar-code-rightPanel-deck').selectedIndex = 
    document.getElementById('solvent-scrapeToolbar-code-rightPanel-tabs').selectedIndex;
}

Code.addTab = function() {
    var codeEditor = document.createElement("codeeditor");
    codeEditor.setAttribute("onrun", "Code.run();");
    
    document.getElementById("solvent-scrapeToolbar-leftPanel-tabpanels").appendChild(codeEditor);
    
    var tabs = document.getElementById("solvent-scrapeToolbar-leftPanel-tabs");
    var v = ++Code._codeTabIndex;
    
    tabs.appendItem(v, v);
    tabs.selectedIndex = tabs.childNodes.length - 1;
    
    document.getElementById("solvent-scrapeToolbar-code-minus-button").collapsed = false; 
}

Code.removeTab = function() {
    if (!window.confirm("Delete currently selected code tab?")) {
        return;
    }
    
    var tabbox = document.getElementById("solvent-scrapeToolbar-leftPanel-tabbox");
    var i = tabbox.selectedIndex;
    var newI = (i == 2) ? 3 : i - 1;
    
    var tabs = document.getElementById("solvent-scrapeToolbar-leftPanel-tabs");
    var tabPanels = document.getElementById("solvent-scrapeToolbar-leftPanel-tabpanels");
    
    tabbox.selectedTab = tabs.childNodes[newI];
    tabbox.selectedPanel = tabPanels.childNodes[newI];
    
    tabs.removeChild(tabs.childNodes[i]);
    tabPanels.removeChild(tabPanels.childNodes[i]);
    
    var newI = (i == 2) ? 2 : i - 1;
    
    tabbox.selectedTab = tabs.childNodes[newI];
    tabbox.selectedPanel = tabPanels.childNodes[newI];
    
    if (tabs.childNodes.length == 3) {
        document.getElementById("solvent-scrapeToolbar-code-minus-button").collapsed = true; 
    }
    
    Code.onCodeTabChanged();
}

Code.onCodeTabChanged = function() {
    var tabs = document.getElementById("solvent-scrapeToolbar-leftPanel-tabs");
    for (var i = 2; i < tabs.childNodes.length; i++) {
        var tab = tabs.childNodes[i];
        if (tab.selected) {
            Code.lastSelectedTabIndex = i - 2;
        }
    }
}

// ----------------------------------------------------------------------

function Code_Buffer() {
    this._indentStep = "  ";
    this._indentLevel = 0;
    this._indent = "";
    this.content = "";
}

Code_Buffer.prototype.append = function(s, indent) {
    if (indent) {
        this.content += this._indent;
    }
    this.content += s;
}

Code_Buffer.prototype.appendLine = function(s, skipIndent) {
    if (!skipIndent) {
        this.content += this._indent;
    }
    this.content += s + "\n";
}

Code_Buffer.prototype.indent = function() {
    this._indentLevel++;
    this._indent += this._indentStep;
}

Code_Buffer.prototype.unindent = function() {
    this._indentLevel--;
    this._indent = "";
    for (var i = 0; i < this._indentLevel; i++) {
        this._indent += this._indentStep;
    }
}

Code.createCodeBuffer = function() {
    return new Code_Buffer();
}
