var DOM = new Object();
DOM._friendlyTree = true;

/*--------------------
 * Regular DOM Tree
 *--------------------
 */
function DOM_TreeItem(node, xpathName, level, index) {
  this.node = node;
  this.xpathName = xpathName;
  this.level = level;
  this.index = index;
  this.isContainer = node.nodeType == 1;
  this.isOpen = false;
  
  var text;
  
  this.labels = [];
  this.labels["class"] = "";
  
  if (node.nodeType == 1) {
    tag = node.tagName.toUpperCase();
    if (tag == "A")
      text = node.href;
    else if (tag == "IMG")
      text = node.src;
    else
      text = "";
      
    var className = node.className;
    if (className && className != "") {
      var classes = className.split(" ");
      var newClasses = [];
      for (var c = 0; c < classes.length; c++) {
        var cl = classes[c];
        if (cl.indexOf("simile-solvent-") < 0)
          newClasses.push(cl);
      }
      if (newClasses.length > 0)
        this.labels["class"] = newClasses.join(" ");
    }
  } else {
    text = node.nodeValue;
  }
  this.labels["tag"] = xpathName + " [" + index + "]";
  this.labels["text"] = text;
}

DOM_TreeItem.prototype.getXPathSegment = function() {
  return this.xpathName + "[" + this.index + "]";
}

function DOM_TreeView(nodes) {
  var rows = [];
  if (nodes) {
    var count = 1;
    for (var i = 0; i < nodes.length; i++) {
      var node = nodes[i];
      var type = node.nodeType;
      var xpathName;
      
      var prefix = node.ownerDocument.documentElement.namespaceURI ? "x:" : "";
      
      if (type == 1) {
        xpathName = prefix + node.tagName;
      } else if (type == 3) {
        xpathName = "text()";
      } else if (type == 8) {
        xpathName = "comment()";
      } else {
        continue; // skip other types of node
      }
      rows.push(new DOM_TreeItem(node, xpathName, 0, count++));
    }
  }
  this._rows = rows;
  this._rootRows = [].concat(rows);
  this._treeBox = null;
}
DOM_TreeView.prototype = {
  setTree : function(treeBox) {
    this._treeBox = treeBox;
  },
  
  
  isContainer : function(row) {
    return this._rows[row].isContainer;
  },
  isContainerOpen : function(row) {
    return this._rows[row].isOpen;
  },
  isContainerEmpty : function(row) {
    return this._rows[row].node.childNodes.length == 0;
  },
  get rowCount() {
    return this._rows.length;
  },
  getCellText : function(row, col) {
    return this._rows[row].labels[this._getColumnId(col)];
  },
  setCellText : function(row, col, text) {
    var colId = this._getColumnId(col);
    if (colId == "var") {
      this._rows[row].labels["var"] = text;
    }
  },
  getParentIndex : function(row) {
    var level = this.getLevel(row);
    for (var r = row - 1; r >= 0; r--) {
      if (this.getLevel(r) < level)
        return r;
    }
    return -1;
  },
  getLevel : function(row) {
    return this._rows[row].level;
  },
  hasNextSibling : function(row, after) {
    var level = this.getLevel(row);
    for (var r = after + 1; r < this._rows.length; r++) {
      var level2 = this.getLevel(r);
      if (level2 == level)
        return true;
      else if (level2 < level)
        break;
    }
    return false;
  },
  
  
  toggleOpenState : function(row) {
    if (this.isContainerEmpty(row))
      return;
      
    var item = this._rows[row];
    if (item.isOpen) {
      // closing item, deleting descendents
      var deleteCount = 0;
      var level = this.getLevel(row);
      for (var r = row + 1; r < this._rows.length; r++) {
        if (this.getLevel(r) > level) {
          deleteCount++;
          this._rows[r].isOpen = false;
        } else {
          break;
        }
      }
      
      if (deleteCount > 0) {
        this._rows.splice(row + 1, deleteCount);
        this._treeBox.rowCountChanged(row + 1, -deleteCount);
      }
    } else {
      var newRows = this._rows[row].childRows;
      if (!newRows) {
        newRows = [];
        
        var prefix = item.node.ownerDocument.documentElement.namespaceURI ? "x:" : "";
        
        // opening item, adding children
        var level = this.getLevel(row) + 1;
        var childNodes = item.node.childNodes;
        var count = childNodes.length;
        var indexMap = [];
        for (var i = 0; i < count; i++) {
          var node = childNodes.item(i);
          var type = node.nodeType;
          var xpathName;
          
          if (type == 1) {
            xpathName = prefix + node.tagName;
          } else if (type == 3) {
            xpathName = "text()";
          } else if (type == 8) {
            xpathName = "comment()";
          } else {
            continue; // skip other types of node
          }
          
          var index = indexMap[xpathName];
          if (!index) {
            index = 1;
          } else {
            index++;
          }
          indexMap[xpathName] = index;
          
          newRows.push(new DOM_TreeItem(node, xpathName, level, index));
        }
        
        // We need to remember variables set on child
        // rows even after they are collapsed
        this._rows[row].childRows = newRows;
      }
      
      this._rows = this._rows.slice(0, row + 1).concat(newRows).concat(this._rows.slice(row + 1));
      this._treeBox.rowCountChanged(row + 1, newRows.length);
    }
    item.isOpen = !item.isOpen;
  },
  
  
  isEditable : function(row, col) {
    return (this.getLevel(row) > 0 && this._getColumnId(col) == "var");
  },
  
  isSeparator :         function(row) { return false; },
  isSorted :             function(row) { return false; },
  getImageSrc :         function(row, col) {},
  getProgressMode :      function(row, col) {},
  getCellValue :        function(row, col) {},
  cycleHeader :          function(colID, elmt) {},
  cycleCell :            function(row, col) {},
  getRowProperties :     function(row, props) {},
  getCellProperties :   function(row, col, props) {},
  getColumnProperties : function(colid, col, props) {},
  selectionChanged :    function() {},
  performAction :        function(action) {},
  performActionOnCell :  function(action, row, col) {},
  
  _getColumnId : function(col) {
    return ((typeof col) == "string") ? col : col.id;
  }
}


/*--------------------
 * Friendly Tree
 *--------------------
 */
function DOM_FriendlyTreeItem(node, index) {
  this.node = node;
  this.level = 0;
  this.isContainer = true;
  this.isOpen = false;
  
  var tag;
  var nodeType = node.nodeType;
  if (nodeType == "1")
    tag = node.tagName;
  else if (nodeType == "3")
    tag = "text";
  else if (nodeType == "8")
    tag = "comment";
  
  this.labels = [];
  this.labels["item"] = "Item " + index + " (" + tag + ")";
  this.labels["text"] = "";
}

function DOM_FriendlyTreeTextItem(node, relativeXPath, text) {
  this.node = node;
  this.level = 1;
  this.isContainer = false;
  this.isOpen = false;
  this.relativeXPath = relativeXPath;
  
  this.labels = [];
  this.labels["item"] = text;
  this.labels["text"] = text;
}

function DOM_FriendlyTreeView(nodes) {
  var rows = [];
  if (nodes) {
    var count = 1;
    for (var i = 0; i < nodes.length; i++) {
      var node = nodes[i];
      var type = node.nodeType;
      if (type == 1 || type == 3 || type == 8) {
        rows.push(new DOM_FriendlyTreeItem(node, count++));
      }
    }
  }
  this._rows = rows;
  this._rootRows = [].concat(rows);
  this._treeBox = null;
}
DOM_FriendlyTreeView.prototype = {
  setTree : function(treeBox) {
    this._treeBox = treeBox;
  },
  
  isContainer : function(row) {
    return this._rows[row].isContainer;
  },
  isContainerOpen : function(row) {
    return this._rows[row].isOpen;
  },
  isContainerEmpty : function(row) {
    return false;
  },
  get rowCount() {
    return this._rows.length;
  },
  getCellText : function(row, col) {
    return this._rows[row].labels[this._getColumnId(col)];
  },
  setCellText : function(row, col, text) {
    var colId = this._getColumnId(col);
    if (colId == "var") {
      this._rows[row].labels["var"] = text;
    }
  },
  getParentIndex : function(row) {
    var level = this.getLevel(row);
    for (var r = row - 1; r >= 0; r--) {
      if (this.getLevel(r) < level)
        return r;
    }
    return -1;
  },
  getLevel : function(row) {
    return this._rows[row].level;
  },
  hasNextSibling : function(row, after) {
    var level = this.getLevel(row);
    for (var r = after + 1; r < this._rows.length; r++) {
      var level2 = this.getLevel(r);
      if (level2 == level)
        return true;
      else if (level2 < level)
        break;
    }
    return false;
  },
  
  
  toggleOpenState : function(row) {
    if (this.isContainerEmpty(row))
      return;
      
    var item = this._rows[row];
    if (item.isOpen) {
      // closing item, deleting descendents
      var deleteCount = 0;
      var level = this.getLevel(row);
      for (var r = row + 1; r < this._rows.length; r++) {
        if (this.getLevel(r) > level) {
          deleteCount++;
          this._rows[r].isOpen = false;
        } else {
          break;
        }
      }
      
      if (deleteCount > 0) {
        this._rows.splice(row + 1, deleteCount);
        this._treeBox.rowCountChanged(row + 1, -deleteCount);
      }
    } else {
      var newRows = this._rows[row].childRows;
      if (!newRows) {
        newRows = [];
        
        var node = item.node;
        var nodeType = item.nodeType;
        
        if (nodeType == 3 || nodeType == 8) { // text and comment
          newRows.push(new DOM_FriendlyTreeTextItem(node, ".", node.nodeValue));
        } else { // element
          var prefix = node.ownerDocument.documentElement.namespaceURI ? "x:" : "";
          
          var f = function(node, relativeXPath, recurse) {
            var tagName = node.tagName;
            if (tagName == "A") {
              newRows.push(new DOM_FriendlyTreeTextItem(node, relativeXPath, node.href));
            } else if (tagName == "IMG") {
              newRows.push(new DOM_FriendlyTreeTextItem(node, relativeXPath, node.src));
            }
            
            var childNodes = node.childNodes;
            var count = childNodes.length;
            var indexMap = [];
            for (var i = 0; i < count; i++) {
              var node2 = childNodes.item(i);
              var type = node2.nodeType;
              var xpathName;
              
              if (type == 1) {
                xpathName = prefix + node2.tagName;
              } else if (type == 3) {
                xpathName = "text()";
              } else if (type == 8) {
                xpathName = "comment()";
              } else {
                continue; // skip other types of node
              }
              
              var index = indexMap[xpathName];
              if (!index) {
                index = 1;
              } else {
                index++;
              }
              indexMap[xpathName] = index;
              
              var xpathSegment = xpathName + "[" + index + "]";
              var thisRelativeXPath = relativeXPath + "/" + xpathSegment;
              if (type == 3 || type == 8) {
                var text = node2.nodeValue;
                if (text.replace(/^\s*$/, "").length > 0)
                  newRows.push(new DOM_FriendlyTreeTextItem(node2, thisRelativeXPath, text));
              } else {
                recurse(node2, thisRelativeXPath, recurse);
              }
            }
          }
          
          f(node, ".", f);
        }
        
        // We need to remember variables set on child
        // rows even after they are collapsed
        this._rows[row].childRows = newRows;
      }
      
      this._rows = this._rows.slice(0, row + 1).concat(newRows).concat(this._rows.slice(row + 1));
      this._treeBox.rowCountChanged(row + 1, newRows.length);
    }
    item.isOpen = !item.isOpen;
  },
  
  
  isEditable : function(row, col) {
    return (this.getLevel(row) > 0 && this._getColumnId(col) == "var");
  },
  
  isSeparator :          function(row) { return false; },
  isSorted :             function(row) { return false; },
  getImageSrc :          function(row, col) {},
  getProgressMode :      function(row, col) {},
  getCellValue :         function(row, col) {},
  cycleHeader :          function(colID, elmt) {},
  cycleCell :            function(row, col) {},
  getRowProperties :     function(row, props) {},
  getCellProperties :    function(row, col, props) {},
  getColumnProperties :  function(colid, col, props) {},
  selectionChanged :     function() {},
  performAction :        function(action) {},
  performActionOnCell :  function(action, row, col) {},
  
  _getColumnId : function(col) {
    return ((typeof col) == "string") ? col : col.id;
  }
}


/*--------------------
 * Rest of Code
 *--------------------
 */
DOM.onDocumentChanged = function() {
  DOM.constructTree(null);
}

DOM.toggleFriendly = function() {
  DOM._friendlyTree = !DOM._friendlyTree;
  document.getElementById("solvent-scrapeToolbar-code-captureTab-domTreeDeck")
    .selectedIndex = DOM._friendlyTree ? 0 : 1;
  document.getElementById("solvent-scrapeToolbar-code-captureTab-friendly-button")
    .setAttribute("friendly", DOM._friendlyTree);
}

DOM.align = function() {
  window.openDialog(
    "chrome://solvent/content/align-dialog.xul", 
    "_blank", 
    "chrome,all,dialog=no,resizable,centerscreen,width=600,height=400", 
    { nodes : DOM._nodes }
  );
}

DOM.constructTree = function(nodes) {
  DOM._nodes = nodes;
  try {
    var view = new DOM_TreeView(nodes);
    var tree = DOM._getTree();
    tree.treeBoxObject.view = view;
      
    if (nodes) {
      var total = nodes.length;
      var treeView = tree.treeBoxObject.view;
      
      for (var i = total; i > 0; i--) {
        treeView.toggleOpenState(view._rows.length - i);
      }
    }
    
    DOM._treeView = view;
  } catch (e) {
    Solvent.debugPrint(e);
  }
  
  try {
    var view = new DOM_FriendlyTreeView(nodes);
    
    DOM._getFriendlyTree().treeBoxObject.view = view;
    
    DOM._friendlyTreeView = view;
  } catch (e) {
    Solvent.debugPrint(e);
  }
}

DOM._onItemVarChanged = function(item) {
  var a = [];
  if (item.isURI) {
    a.push("URI");
  }
  if (item.isTitle) {
    a.push("Title");
  }
  if (item.isDescription) {
    a.push("Desc");
  }
  if (item.isAddress) {
    a.push("Addr");
  }
  if (item.customProperty) {
    a.push(item.customProperty);
  }
  if (item.customVariable) {
    a.push(item.customVariable);
  }
  if (item.code && item.code.length > 0) {
    a.push(item.code);
  }
  item.labels["var"] = a.join(" | ");
  
  var tree = DOM._friendlyTree ? DOM._getFriendlyTree() : DOM._getTree();
  tree.treeBoxObject.invalidateRow(tree.currentIndex);
}

DOM.nameItemURI = function() {
  var item = DOM._getSelectedItemObject();
  if (item.labels["text"].length == 0) {
    window.alert("This element has no text that can be used as a URI.");
  } else if (!item.isURI) {
    item.isURI = true;
    DOM._onItemVarChanged(item);
  }
}

DOM.nameItemTitle = function() {
  var item = DOM._getSelectedItemObject();
  if (item.labels["text"].length == 0) {
    window.alert("This element has no text that can be used as a title.");
  } else if (!item.isTitle) {
    item.isTitle = true;
    DOM._onItemVarChanged(item);
  }
}

DOM.nameItemDescription = function() {
  var item = DOM._getSelectedItemObject();
  if (item.labels["text"].length == 0) {
    window.alert("This element has no text that can be used as a description.");
  } else if (!item.isDescription) {
    item.isDescription = true;
    DOM._onItemVarChanged(item);
  }
}

DOM.nameItemAddress = function() {
  var item = DOM._getSelectedItemObject();
  if (item.labels["text"].length == 0) {
    window.alert("This element has no text that can be used as an address.");
  } else if (!item.isAddress) {
    item.isAddress = true;
    DOM._onItemVarChanged(item);
  }
}

DOM.nameItemCustomProperty = function() {
  var item = DOM._getSelectedItemObject();
  if (item.labels["text"].length == 0) {
    window.alert("This element has no text that can be used as a custom property.");
  } else {
    var property = window.prompt("Enter property URI", "", "Custom Property");
    if (property && property != "") {
      item.customProperty = property;
      DOM._onItemVarChanged(item);
    }
  }
}

DOM.nameItemCustomVariable = function() {
  var item = DOM._getSelectedItemObject();
  var variable = window.prompt("Enter variable name", "", "Custom Variable");
  if (variable && variable != "") {
    item.customVariable = variable;
    DOM._onItemVarChanged(item);
  }
}

DOM.processText = function() {
  var item = DOM._getSelectedItemObject();
  var text = item.labels["text"];
  if (text) {
    var o = { text : text, code : item.code };
    window.openDialog(
      "chrome://solvent/content/process-text-dialog.xul", 
      "process-text", 
      "chrome,modal,resizable,centerscreen,width=400,height=400", 
      o);
      
    item.code = o.code;
    DOM._onItemVarChanged(item);
  }
}

DOM._getSelectedItemObject = function() {
  if (DOM._friendlyTree) {
    var tree = DOM._getFriendlyTree();
    var i = tree.currentIndex;
    return DOM._friendlyTreeView._rows[i];
  } else {
    var tree = DOM._getTree();
    var i = tree.currentIndex;
    return DOM._treeView._rows[i];
  }
}

DOM.onTreeSelect = function() {
  var i = DOM._getTree().currentIndex;
  var treeItem = DOM._treeView._rows[i];
  Capture.highlightNode(treeItem.node);
}
DOM.onFriendlyTreeSelect = function() {
  var i = DOM._getFriendlyTree().currentIndex;
  var treeItem = DOM._friendlyTreeView._rows[i];
  Capture.highlightNode(treeItem.node);
}

DOM._getTree = function() {
  return document.getElementById('solvent-scrapeToolbar-code-captureTab-dom-tree');
}
DOM._getFriendlyTree = function() {
  return document.getElementById('solvent-scrapeToolbar-code-captureTab-friendly-tree');
}

DOM.generateJavascript = function() {
  // Gather information about variables to generate in code
  //
  var uriItems = [];
  var titleItems = [];
  var descriptionItems = [];
  var addressItems = [];
  var customPropertyItems = [];
  var customVariableItems = [];

  var pushItem = function(item) {
    if (item.isURI)
      uriItems.push(item);
    if (item.isTitle)
      titleItems.push(item);
    if (item.isDescription)
      descriptionItems.push(item);
    if (item.isAddress)
      addressItems.push(item);
    if (item.customProperty)
      customPropertyItems.push(item);
    if (item.customVariable)
      customVariableItems.push(item);
  }
  
  if (DOM._friendlyTree) {
    var rootRows = DOM._friendlyTreeView._rootRows;
    for (var i = 0; i < rootRows.length; i++) {
      var childRows = rootRows[i].childRows;
      if (childRows) {
        for (var j = 0; j < childRows.length; j++) {
          pushItem(childRows[j]);
        }
      }
    }
  } else {
    var f = function(items, recurse, parentXPath) {
      for (var i = 0; i < items.length; i++) {
        var item = items[i];
        pushItem(item);
        if (item.level == 0)
          item.relativeXPath = ".";
        else
          item.relativeXPath = parentXPath + "/" + item.xpathName + "[" + item.index + "]";
          
        if (item.childRows)
          recurse(item.childRows, recurse, item.relativeXPath);
      }
    }
    f(DOM._treeView._rootRows, f, ".");
  }
  
  var codeBuffer = Code.createCodeBuffer();
  var utilitiesBuffer = Code.createCodeBuffer();
  
  var generateGetNodeFunction = false;
  var generateCleanStringFunction = false;
  
  // Code generating utilities
  //
  var generateGetNodeCode = function(item, docVariableName, contextNodeVariableName) {
    generateGetNodeFunction = true;
    if (item.level == 0)
      return contextNodeVariableName;
    else
      return "getNode(" + docVariableName + ", " + 
        contextNodeVariableName + ", '" + item.relativeXPath + "', nsResolver)";
  };
  var generateCleanUpStringCode = function(stringCode) {
    generateCleanStringFunction = true;
    return "cleanString(" + stringCode + ")";
  }
  var generateGetTextCode = function(cb, item, docVariableName, contextNodeVariableName) {
    var suffix;
    if (item.node.nodeType == 1) {
      var tag = item.node.tagName.toUpperCase();
      if (tag == "A") {
        suffix = ".href";
      } else if (tag == "IMG") {
        suffix = ".src";
      } else {
        suffix = ".innerHTML";
      }
    } else {
      suffix = ".nodeValue";
    }
    
    var code = generateGetNodeCode(item, docVariableName, contextNodeVariableName) + suffix;
    
    var customCode = item.code;
    if (customCode && customCode.length > 0) {
      cb.appendLine("function(input) {", true);
        cb.indent();
        var lines = customCode.split("\n");
        for (var l = 0; l < lines.length; l++) {
          cb.appendLine(lines[l]);
        }
        cb.appendLine("return output;");
        cb.unindent();
      cb.append("}(" + code + ")", true);
    } else {
      cb.append(generateCleanUpStringCode(code));
    }
  };
  var generateExceptionHandlingCode = function() {
    return("utilities.debugPrint(e);");
  };
  
  // Generate the code
  //
  codeBuffer.appendLine("var xpath = '" + Capture.getXPathTextbox().value + "';");
  codeBuffer.appendLine("var elmts = utilities.gatherElementsOnXPath(doc, doc, xpath, nsResolver);");
  codeBuffer.appendLine("for (var i = 0; i < elmts.length; i++) {");
    codeBuffer.indent();
    codeBuffer.appendLine("var elmt = elmts[i];");
    codeBuffer.appendLine("// elmt.style.backgroundColor = 'red';");
    codeBuffer.appendLine("");
    
    // Generate code to retrieve URI
    //
    if (uriItems.length > 0) {
      codeBuffer.appendLine("var uri;");
      for (var i = 0; i < uriItems.length; i++) {
        var item = uriItems[i];
        
        codeBuffer.appendLine("try {");
          codeBuffer.indent();
          codeBuffer.append("uri = ", true);
            generateGetTextCode(codeBuffer, item, "doc", "elmt");
            codeBuffer.appendLine(";", true);
          codeBuffer.unindent();
        codeBuffer.appendLine("} catch (e) { " + generateExceptionHandlingCode() + "}");
      }
    } else {
      codeBuffer.appendLine("var uri = 'item' + i; // generate the item's URI here");
    }
    codeBuffer.appendLine("");
    
    // Generate code to assert type
    //
    codeBuffer.appendLine("model.addStatement(uri, prefixRDF + 'type', 'unknown', false); // Use your own type here");
    codeBuffer.appendLine("// utilities.debugPrint('Scraping URI ' + uri);");
    codeBuffer.appendLine("");
    
    // Generate code to assert title
    //
    if (titleItems.length > 0) {
      for (var i = 0; i < titleItems.length; i++) {
        var item = titleItems[i];
        
        codeBuffer.appendLine("try {");
          codeBuffer.indent();
          codeBuffer.append("model.addStatement(uri, prefixDC + 'title', ", true);
            generateGetTextCode(codeBuffer, item, "doc", "elmt")
            codeBuffer.appendLine(", true);", true);
          codeBuffer.unindent();
        codeBuffer.appendLine("} catch (e) { " + generateExceptionHandlingCode() + "}");
      }
      codeBuffer.appendLine("");
    }
    
    // Generate code to assert description
    //
    if (descriptionItems.length > 0) {
      for (var i = 0; i < descriptionItems.length; i++) {
        var item = descriptionItems[i];
        
        codeBuffer.appendLine("try {");
          codeBuffer.indent();
          codeBuffer.append("model.addStatement(uri, prefixDC + 'description', ", true);
            generateGetTextCode(codeBuffer, item, "doc", "elmt");
            codeBuffer.appendLine(", true);", true);
          codeBuffer.unindent();
        codeBuffer.appendLine("} catch (e) { " + generateExceptionHandlingCode() + "}");
      }
      codeBuffer.appendLine("");
    }
    
    // Generate code to assert address
    //
    if (addressItems.length > 0) {
      codeBuffer.appendLine("var addressSegments = [];");
      for (var i = 0; i < addressItems.length; i++) {
        var item = addressItems[i];
        
        codeBuffer.appendLine("try {");
          codeBuffer.indent();
          codeBuffer.append("addressSegments.push(", true);
            generateGetTextCode(codeBuffer, item, "doc", "elmt");
            codeBuffer.appendLine(");", true);
          codeBuffer.unindent();
        codeBuffer.appendLine("} catch (e) { " + generateExceptionHandlingCode() + "}");
      }
      codeBuffer.appendLine("var address = addressSegments.join(', ');");
      codeBuffer.appendLine("model.addStatement(uri, prefixLoc + 'address', address, true);");
      codeBuffer.appendLine("geo.add(uri, address);");
      codeBuffer.appendLine("");
    }
    
    // Generate code for custom properties
    //
    if (customPropertyItems.length > 0) {
      for (var i = 0; i < customPropertyItems.length; i++) {
        var item = customPropertyItems[i];
        var customProperty = item.customProperty;
        if (customProperty.indexOf("'") < 0 && customProperty.indexOf('"') < 0)
          customProperty = "'" + customProperty + "'";
        
        codeBuffer.appendLine("try {");
          codeBuffer.indent();
          codeBuffer.append("model.addStatement(uri, " + customProperty + ", ", true);
            generateGetTextCode(codeBuffer, item, "doc", "elmt");
            codeBuffer.appendLine(", true);", true);
          codeBuffer.unindent();
        codeBuffer.appendLine("} catch (e) { " + generateExceptionHandlingCode() + "}");
        codeBuffer.appendLine("");
      }
    }
    
    // Generate code for custom variables
    //
    if (customVariableItems.length > 0) {
      for (var i = 0; i < customVariableItems.length; i++) {
        var item = customVariableItems[i];
        var customVariable = item.customVariable;
        
        codeBuffer.appendLine("try {");
          codeBuffer.indent();
          codeBuffer.append("var " + customVariable + " = ", true);
            generateGetTextCode(codeBuffer, item, "doc", "elmt");
            codeBuffer.appendLine(";", true);
          codeBuffer.unindent();
        codeBuffer.appendLine("} catch (e) { " + generateExceptionHandlingCode() + "}");
        codeBuffer.appendLine("");
      }
    }
    
    codeBuffer.unindent();
  codeBuffer.appendLine("}");

  // Generate ontology prefixes
  //
  utilitiesBuffer.appendLine("var prefixRDF = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#';");
  utilitiesBuffer.appendLine("var prefixDC = 'http://purl.org/dc/elements/1.1/';");
  if (addressItems.length > 0)
    utilitiesBuffer.appendLine("var prefixLoc = 'http://simile.mit.edu/2005/05/ontologies/location#';");
  utilitiesBuffer.appendLine("");
  
  // Generate utility functions
  //
  utilitiesBuffer.appendLine("var namespace = doc.documentElement.namespaceURI;");
  utilitiesBuffer.appendLine("var nsResolver = namespace ? function(prefix) {");
    utilitiesBuffer.indent();
    utilitiesBuffer.appendLine("if (prefix == 'x') return namespace; else return null;");
    utilitiesBuffer.unindent();
  utilitiesBuffer.appendLine("} : null;");
  utilitiesBuffer.appendLine("");
  
  if (generateGetNodeFunction) {
    utilitiesBuffer.appendLine("var getNode = function(doc, contextNode, xpath, nsResolver) {");
      utilitiesBuffer.indent();
      utilitiesBuffer.appendLine("return doc.evaluate(xpath, contextNode, nsResolver, XPathResult.ANY_TYPE,null).iterateNext();");
      utilitiesBuffer.unindent();
    utilitiesBuffer.appendLine("}");
    utilitiesBuffer.appendLine("");
  }
  if (generateCleanStringFunction) {
    utilitiesBuffer.appendLine("var cleanString = function(s) {");
      utilitiesBuffer.indent();
      utilitiesBuffer.appendLine("return utilities.trimString(s);");
      utilitiesBuffer.unindent();
    utilitiesBuffer.appendLine("}");
    utilitiesBuffer.appendLine("");
  }  
  if (addressItems.length > 0) {
    utilitiesBuffer.appendLine("var onAddressLookedUp = function (itemURI, latlong) {");
      utilitiesBuffer.indent();
        utilitiesBuffer.appendLine("model.addStatement(itemURI, prefixLoc + 'coordinates', latlong, true);");
      utilitiesBuffer.unindent();
    utilitiesBuffer.appendLine("}");
    utilitiesBuffer.appendLine("var geo = utilities.createGeoHelper(onAddressLookedUp, utilities);");
    utilitiesBuffer.appendLine("");
    
    codeBuffer.appendLine("");
    codeBuffer.appendLine("wait(); // don't navigate to collected data just yet, let the address lookup begin");
    codeBuffer.appendLine("geo.lookupAddresses(done, done);");
  }
  
  return utilitiesBuffer.content + codeBuffer.content;
}
