var Highlighter = new Object();

Highlighter._nodeLists = [];
Highlighter._markerMaps = [];

Highlighter._get = function(a, x) {
    var m = a[x];
    if (!m) {
        m = [];
        a[x] = m;
    }
    return m;
}

Highlighter._getNodeList = function(styleClass) {
    return Highlighter._get(Highlighter._nodeLists, styleClass);
}
Highlighter._getMarkerMap = function(styleClass) {
    return Highlighter._get(Highlighter._markerMaps, styleClass);
}

Highlighter.highlight = function(node, styleClass, markerStyler) {
    if (node.nodeType != 1) {
        return; // can't highlight non-element
    }
    
    var className = node.className;
    if (className == "") {
        node.className = styleClass;
    } else if (className.indexOf(styleClass) < 0) {
        node.className = className + " " + styleClass;
    }
    
    var top = 0;
    var node2 = node;
    while (node2) {
        top += node2.offsetTop;
        node2 = node2.offsetParent;
    }
    top = Math.round(top * 100 / node.ownerDocument.body.offsetHeight) + "%";
    
    var markerDiv = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
    markerDiv.style.position = "absolute";
    markerDiv.style.top = top;
    markerStyler(markerDiv);
    
    var body = document.getElementById("solvent-gutter").contentDocument.body;
    body.appendChild(markerDiv);
    
    var solventKey = Math.floor(Math.random() * 1000000);
    node.setAttribute("solvent-key-" + styleClass, solventKey);
    
    Highlighter._getNodeList(styleClass).unshift(node);
    Highlighter._getMarkerMap(styleClass)[solventKey] = markerDiv;
};

Highlighter.unhighlight = function(node, styleClass) {
    var nodes = Highlighter._getNodeList(styleClass);
    var count = nodes.length;
    
    for (var i = 0; i < count; i++) {
        var node2 = nodes[i];
        if (node2  == node) {
            Highlighter._removeStyleClass(node, styleClass);
            Highlighter._removeMarker(node, styleClass);
            nodes.splice(i, 1);
            break;
        }
    }
}

Highlighter.unhighlightStyle = function(styleClass) {
    var nodes = Highlighter._getNodeList(styleClass);
    var count = nodes.length;
    
    for (var i = 0; i < count; i++) {
        var node = nodes[i];
        Highlighter._removeStyleClass(node, styleClass);
        Highlighter._removeMarker(node, styleClass);
    }
    
    Highlighter._nodeLists[styleClass] = [];
}

Highlighter._removeStyleClass = function(node, styleClass) {
    var className = node.className;
    var n = className.indexOf(styleClass);
    if (n >= 0) {
        className = className.substring(0, n) + className.substring(n + styleClass.length);
        node.className = className;
    }
};

Highlighter._removeMarker = function(node, styleClass) {
    try {
        var solventKey = node.getAttribute("solvent-key-" + styleClass);
        var markerMap = Highlighter._getMarkerMap(styleClass);
        var markerDiv = markerMap[solventKey];
        markerMap[solventKey] = null;
        
        markerDiv.parentNode.removeChild(markerDiv);
    } catch (e) {
        Solvent.debugPrint(e);
    }
};

