var Results = new Object();

function Results_StatementItem(predicateURI, object) {
    this.labels = [];
    this.labels["uri"] = '<' + predicateURI + '>';
    this.labels["object"] = object;
    this.isContainer = false;
    this.isContainerEmpty = true;
    this.isOpen = false;
}

function Results_SubjectItem(subjectURI, statementItems, tags) {
    this.labels = [];
    this.labels["uri"] = '<' + subjectURI + '>';
    this.labels["object"] = tags.join(',');
    this.isContainer = true;
    this.isContainerEmpty = statementItems.length == 0;
    this.isOpen = true;
    this.statementItems = statementItems;
}

function Results_TreeView(items) {
    this._rows = items;
    this._treeBox = null;
}

Results_TreeView.prototype = {

    get rowCount() {
        return this._rows.length;
    },

    setTree : function(treeBox) {
        this._treeBox = treeBox;
    },
    isContainer : function(row) {
        return this._rows[row].isContainer;
    },
    isContainerOpen : function(row) {
        return this._rows[row].isOpen;
    },
    isContainerEmpty : function(row) {
        return this._rows[row].isContainerEmpty;
    },
    getCellText : function(row, col) {
        return this._rows[row].labels[((typeof col) == "string") ? col : col.id];
    },
    getParentIndex : function(row) {
        var level = this.getLevel(row);
        for (var r = row - 1; r >= 0; r--) {
            if (this.getLevel(r) < level) {
                return r;
            }
        }
        return -1;
    },
    getLevel : function(row) {
        return this._rows[row].isContainer ? 0 : 1;
    },
    hasNextSibling : function(row, after) {
        var level = this.getLevel(row);
        for (var r = after + 1; r < this._rows.length; r++) {
            var level2 = this.getLevel(r);
            if (level2 == level) {
                return true;
            } else if (level2 < level) {
                break;
            }
        }
        return false;
    },
  
    toggleOpenState : function(row) {
        if (this.isContainerEmpty(row)) {
            return;
        }
    
        var item = this._rows[row];
        if (item.isOpen) {
            // closing item, deleting descendents
            var deleteCount = 0;
            var level = this.getLevel(row);
            for (var r = row + 1; r < this._rows.length; r++) {
                if (this.getLevel(r) > level) {
                  deleteCount++;
                } else {
                  break;
                }
            }
    
            if (deleteCount > 0) {
                this._rows.splice(row + 1, deleteCount);
                this._treeBox.rowCountChanged(row + 1, -deleteCount);
            }
        } else {
            // opening item, adding children
            var statementItems = item.statementItems;
            this._rows = this._rows.slice(0, row + 1).concat(statementItems).concat(this._rows.slice(row + 1));      
            this._treeBox.rowCountChanged(row + 1, statementItems.length);
        }
        item.isOpen = !item.isOpen;
    },
  
    isSeparator :          function(row) { return false; },
    isSorted :             function(row) { return false; },
    isEditable :           function(row, col) { return false; },
    getImageSrc :          function(row, col) {},
    getProgressMode :      function(row, col) {},
    getCellValue :         function(row, col) {},
    cycleHeader :          function(colID, elmt) {},
    cycleCell :            function(row, col) {},
    getRowProperties :     function(row, props) {},
    getCellProperties :    function(row, col, props) {},
    getColumnProperties :  function(colid, col, props) {},
    selectionChanged :     function() {},
    performAction :        function(action) {},
    performActionOnCell :  function(action, row, col) {}
}

Results.setModel = function(model) {
    var subjectList = model.subjectList;
    var subjectToStatements = model.subjectToStatements;
    var subjectToTags = model.subjectToTags;
    
    var items = [];
    var subjectItems = [];
    for (var i = 0; i < subjectList.length; i++) {
        var subject = subjectList[i];
        var statements = subjectToStatements[subject];
        var tags = subjectToTags[subject];
        
        var statementItems = [];
        for (var s = 0; s < statements.length; s++) {
            var statement = statements[s];
            var statementItem = new Results_StatementItem(statement.predicate, statement.object);
            
            statementItems.push(statementItem);
        }
        
        var subjectItem = new Results_SubjectItem(subject, statementItems, tags);
        subjectItems.push(subjectItem);
        
        items.push(subjectItem);
        items = items.concat(statementItems);
    }
    
    Results._getTree().treeBoxObject.view = new Results_TreeView(items);
    
    Results._subjectItems = subjectItems;
    Results.count = subjectItems.length;
    
    var tab = document.getElementById("solvent.scrapeToolbar.code.resultsTab");
    if (Results.count > 0) {
        tab.setAttribute("label", "Results (" + Results.count + ")");
    } else {
        tab.setAttribute("label", "Results");
    }
    
    Results._writeResultsToFile();
}

Results.onTreeSelect = function() {
    var tree = Results._getTree();
    var columns = tree.columns;
    var view = tree.treeBoxObject.view;
    var i = tree.currentIndex;
    
    var text;
    if (view.isContainer(i)) {
        text = view.getCellText(i, columns.getColumnAt(0));
    } else {
        text = view.getCellText(i, columns.getColumnAt(1));
    }
    
    document.getElementById('solvent-scrapeToolbar-resultsTab-textbox').value = text;
}

Results._getResultsFile = function() {
    return Solvent.chromeURLToFile("chrome://solvent/content/results.n3");
}

Results._writeResultsToFile = function() {
    if (!Results._subjectItems) {
        return;
    }
    
    try {
        var file = Results._getResultsFile();
        try {
            file.remove(false);
        } catch (e) {
            Solvent.debugPrint("Can't remove file: " + e);
        }
        
        var fo = Solvent.openUTF8FileToWrite(file);
        
        for (var i = 0; i < Results._subjectItems.length; i++) {
            var subjectItem = Results._subjectItems[i];
            var statementItems = subjectItem.statementItems;
            if (statementItems.length == 0) {
                continue;
            }
            
            var t = subjectItem.labels["uri"] + "\n";
            fo.write(t);
            
            for (var s = 0; s < statementItems.length; s++) {
                var statementItem = statementItems[s];
                
                if (s == 0) {
                    t = "  ";
                } else {
                    t = "; ";
                }
            
                t += statementItem.labels["uri"] + " " + statementItem.labels["object"] + "\n";
                fo.write(t);
            }
            
            fo.write(".\n\n");
        }
        fo.close();
    } catch (e) {
        Solvent.debugPrint(e);
    }
}

Results.showInPiggyBank = function() {
      if (!Results._subjectItems) {
        return;
    }
    
    window.openDialog(
        "chrome://browser/content/browser.xul", 
        "_blank", 
        "chrome,all,dialog=no", 
        Results._getResultsFile().path, 
        null, 
        null
    );
}

Results._getTree = function() {
    return document.getElementById('solvent-scrapeToolbar-code-resultsTab-tree');
}
