/*
 *  Constants
 */
const MODE_WRONLY   = 0x02;
const MODE_CREATE   = 0x08;
const MODE_APPEND   = 0x10;
const MODE_TRUNCATE = 0x20;

const PERMS_FILE      = 0644;
const PERMS_DIRECTORY = 0755;

/*
 * The Solvent object skeleton
 */
var Solvent = new Object();

function Solvent_SaveSettings() {
    this.scraperTitle = "";
    this.scraperURI = "";
    this.scraperDescription = "";
    
    this.authorName = "";
    this.codeBufferList = [];
    this.codeBufferIndex = 0;
    this.codeBufferRetriever = function(index) {
        return document.getElementById("solvent-scrapeToolbar-leftPanel-tabpanels").childNodes[index + 2].codeTextbox.value;
    };
    
    this.urlPatterns = [];
    
    this.codeFile = "";
    this.metadataFile = "";
    
    this.publish = false;
    this.publishURL = "";
}

Solvent._firstTime = true;
Solvent._saveSettings = new Solvent_SaveSettings();

Solvent.openUI = function() {
    document.getElementById("solvent-scrapeToolbar-leftPanel-tabs").selectedIndex = 2;
    Solvent._setHidden(false);
    
    if (Solvent._firstTime) {
        Solvent._firstTime = false;
    } else if (Solvent._saveSettings.codeFile != "") {
        if (window.confirm("Discard settings of previous scrapers?")) {
            Solvent._saveSettings = new Solvent_SaveSettings();
        }
    }
};

Solvent.closeUI = function() {
    Solvent._setHidden(true);
};

Solvent._previousLeftPanelTabIndex = -1;

Solvent.onLeftPanelTabSelected = function() {
    var tabs = document.getElementById('solvent-scrapeToolbar-leftPanel-tabs');
    var i = tabs.selectedIndex;
    
    if (i == 1) { // separator
        if (Solvent._previousLeftPanelTabIndex < i) {
            i++;
        } else {
            i--;
            tabs.selectedIndex = i;
        }
    } else {
        Solvent._previousLeftPanelTabIndex = i;
        
        var rightPanelDeck = document.getElementById('solvent-scrapeToolbar-rightPanel-deck');
        var toolbarsDeck = document.getElementById('solvent-scrapeToolbar-leftPanel-toolbars-deck');
        var splitter = document.getElementById('solvent-scrapeToolbar-panel-splitter');
        
        var isCode = (i > 1);
        
        rightPanelDeck.hidden = !isCode;
        splitter.hidden = !isCode;
        
        toolbarsDeck.selectedIndex = isCode ? 1 : 0;
        
        Code.onCodeTabChanged();
    }
};

Solvent._setHidden = function(hidden) {
    document.getElementById("solvent-scrapeToolbar-splitter").hidden = hidden;
    document.getElementById("solvent-scrapeToolbar").hidden = hidden;
    document.getElementById("solvent-gutter").hidden = hidden;
    document.getElementById("solvent-status-panel").hidden = !hidden;
};

Solvent.debugPrint = function (msg) {
    Components.classes["@mozilla.org/consoleservice;1"]
        .getService(Components.interfaces.nsIConsoleService)
            .logStringMessage(msg);
};

Solvent.chromeURLToFile = function(url) {
    var chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIChromeRegistry);
    var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);

    uri.spec = url;
    
    var path = chromeRegistry.convertChromeURL(uri);
    var file;
    
    if (typeof(path) == "object") {
        file = path.QueryInterface(Components.interfaces.nsIFileURL).file;
    } else {
        path = new java.io.File(new Packages.java.net.URI(path)).getAbsolutePath();
        
        file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
        file.initWithPath(path);
    }
    
    return file;
}

Solvent.getFile = function(path) {
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(path);
    return file;
}

Solvent.openFileToWrite = function(file) {
    var fos = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
    
    fos.init(file, MODE_WRONLY | MODE_CREATE, PERMS_FILE, 0);
    
    return {
        _fos : fos,
        write : function(s) {
            this._fos.write(s, s.length);
        },
        close : function() {
            this._fos.close();
        }
    }
}

Solvent.openUTF8FileToWrite = function(file) {
    var fos = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
        
    fos.init(file, MODE_WRONLY | MODE_CREATE, PERMS_FILE, 0);
    
    var unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    unicodeConverter.charset = "utf-8";
  
    return {
        _fos : fos,
        _unicodeConverter : unicodeConverter,
        
        write : function(s) {
            s = this._unicodeConverter.ConvertFromUnicode(s);
            this._fos.write(s, s.length);
        },
        close : function() {
            this._fos.close();
        }
    };
}

Solvent.readFromFile = function(file) {
    var fis = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
    fis.init(file, -1, -1, 0);
    
    var lis = fis.QueryInterface(Components.interfaces.nsILineInputStream);
    
    var s = "";
    var line = { value: "" };
    var more = false;
    do {
        more = lis.readLine(line);
        if (line.value) {
            s += line.value + "\n";
        } else {
            s += "\n";
        }
    } while (more);
    fis.close();
    
    return s;
}

Solvent.save = function() {
    var codeBuffers = [];
    var urlPatterns = [];
    
    var tabs = document.getElementById("solvent-scrapeToolbar-leftPanel-tabs").childNodes;
    for (var i = 2; i < tabs.length; i++) {
        var value = tabs[i].getAttribute("value");
        codeBuffers.push(value);
    }
    
    var listbox = document.getElementById("solvent-scrapeToolbar-urls-listbox");
    for (i = 0; i < listbox.getRowCount(); i++) {
        var s = listbox.getItemAtIndex(i).firstChild.value;
        if (s != "") {
            urlPatterns.push(s);
        }
    }
    
    if (urlPatterns.length == 0) {
        if (!window.confirm("You have specified no URL patterns. Your scraper would not work in Piggy Bank. Continue to save?")) {
            return;
        }
    }
    
    Solvent._saveSettings.codeBufferList = codeBuffers;
    Solvent._saveSettings.codeBufferIndex = Code.lastSelectedTabIndex;
    Solvent._saveSettings.urlPatterns = urlPatterns;
    
    window.openDialog(
        "chrome://solvent/content/save-dialog.xul", 
        "solvent-save", 
        "chrome,modal,resizable,centerscreen,width=450,height=350", 
        Solvent._saveSettings
    );
}
